//----------------------------------------------------------------------
// PennyDesktopRunnable.java
// Author: Bob Dondero
//----------------------------------------------------------------------

package edu.princeton.penny;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.HttpURLConnection;
import java.util.ArrayList;
import org.json.JSONObject;
import org.json.JSONArray;

//----------------------------------------------------------------------

public class PennyDesktopRunnable implements Runnable
{
   private String serverURL;
   private JTextField authorTextField = new JTextField("");
   private DefaultListModel bookListModel = new DefaultListModel();

   //-------------------------------------------------------------------
   public PennyDesktopRunnable(String serverURL)
   {
      this.serverURL = serverURL;
   }
   //-------------------------------------------------------------------

   private class AuthorDocumentListener implements DocumentListener
   {
      private void update()
      {
         String author = authorTextField.getText();

         try
         {
            URL url = new URL(serverURL +
                              "/searchresults?author=" + author);
            HttpURLConnection con =
               (HttpURLConnection) url.openConnection();

            con.setRequestMethod("GET");
            con.setDoInput(true);
            con.connect();
            int responseCode = con.getResponseCode();
            if (responseCode != HttpURLConnection.HTTP_OK)
               throw new IOException(
                  "HTTP error code: " + responseCode);

            BufferedReader in =
               new BufferedReader(
                  new InputStreamReader(con.getInputStream()));
            String jsonDoc = in.readLine();
            in.close();

            ArrayList<String> bookList = new ArrayList<String>();
            JSONArray jsonArray = new JSONArray(jsonDoc);
            for (int i = 0; i < jsonArray.length(); i++)
            {
               JSONObject jsonObject = jsonArray.getJSONObject(i);
               bookList.add(
                  jsonObject.getString("isbn")
                  + ": " + jsonObject.getString("author")
                  + ": " + jsonObject.getString("title")
               );
            }

            bookListModel.removeAllElements();
            for (String book: bookList)
               bookListModel.addElement(book);
         }
         catch (Exception e)
         {
            bookListModel.removeAllElements();
            bookListModel.addElement(e.toString());
         }
      }

      public void changedUpdate(DocumentEvent event) { update(); }
      public void removeUpdate(DocumentEvent event) { update(); }
      public void insertUpdate(DocumentEvent event) { update(); }
   }

   //-------------------------------------------------------------------

   public void run()
   {
      JLabel authorLabel = new JLabel("Author:  ", JLabel.RIGHT);
      JList bookList = new JList(bookListModel);

      JPanel inputPanel = new JPanel();
      inputPanel.setLayout(new BorderLayout());
      inputPanel.add("West", authorLabel);
      inputPanel.add("Center", authorTextField);

      JFrame frame = new JFrame();
      frame.setTitle("Penny");
      frame.setLayout(new BorderLayout());
      frame.add("North", inputPanel);
      frame.add("Center", bookList);
      Toolkit kit = Toolkit.getDefaultToolkit();
      Dimension screenSize = kit.getScreenSize();
      int screenWidth = screenSize.width;
      int screenHeight = screenSize.height;
      frame.setSize(screenWidth/4, screenHeight/4);

      AuthorDocumentListener authorDocumentListener =
         new AuthorDocumentListener();
      authorTextField.getDocument().
         addDocumentListener(authorDocumentListener);

      frame.setLocationByPlatform(true);
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setVisible(true);
   }
}
