//----------------------------------------------------------------------
// WorkerThread.java
// Author: Bob Dondero
//----------------------------------------------------------------------

package edu.princeton.penny;

import javax.swing.DefaultListModel;
import javax.swing.SwingUtilities;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.HttpURLConnection;
import java.util.ArrayList;
import org.json.JSONObject;
import org.json.JSONArray;

//----------------------------------------------------------------------

class WorkerThread extends Thread
{
   private String serverURL;
   private String author;
   private DefaultListModel bookListModel;

   //-------------------------------------------------------------------

   public WorkerThread(String serverURL, String author,
      DefaultListModel bookListModel)
   {
      this.serverURL = serverURL;
      this.author = author;
      this.bookListModel = bookListModel;
   }

   //-------------------------------------------------------------------

   public void run()
   {
      try
      {
         URL url = new URL(serverURL +
                           "/searchresults?author=" + author);
         HttpURLConnection con =
            (HttpURLConnection) url.openConnection();

         con.setRequestMethod("GET");
         con.setDoInput(true);
         con.connect();
         int responseCode = con.getResponseCode();
         if (responseCode != HttpURLConnection.HTTP_OK)
            throw new IOException(
               "HTTP error code: " + responseCode);

         BufferedReader in =
            new BufferedReader(
               new InputStreamReader(con.getInputStream()));
         String jsonDoc = in.readLine();
         in.close();

         ArrayList<String> bookList = new ArrayList<String>();
         JSONArray jsonArray = new JSONArray(jsonDoc);
         for (int i = 0; i < jsonArray.length(); i++)
         {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            bookList.add(
               jsonObject.getString("isbn")
               + ": " + jsonObject.getString("author")
               + ": " + jsonObject.getString("title")
            );
         }

         SwingUtilities.invokeLater(
            () -> {
               bookListModel.removeAllElements();
               for (String book: bookList)
                  bookListModel.addElement(book);
            }
         );
      }
      catch (Exception e)
      {
         SwingUtilities.invokeLater(
            () -> {
               bookListModel.removeAllElements();
               bookListModel.addElement(e.toString());
            }
         );
      }
   }
}
